@@ -80,6 +80,43 @@ class Server(socketserver.ThreadingMixIn,
     # profiling.
     clock_class = stats.NullClock
 
+    def server_bind(self):
+        """Override server_bind to create socket with correct family."""
+        host, port = self.server_address
+
+        # In the past we use "" as a special address to bind to all interfaces.
+        # Using "" with socket.getaddrinfo() would result into socket.gaierror.
+        # To keep backward compatibility with old configurations, replace ""
+        # with "0" which acts in the same way and will be bound to IPv4
+        # interface.
+        if host == "":
+            host = "0"
+
+        addresses = socket.getaddrinfo(host, port, type=self.socket_type)
+        log.debug("Available network interfaces: %s", addresses)
+        self.address_family = addresses[0][0]
+
+        # Close old socket created in constructor first.
+        if self.socket:
+            self.socket.close()
+
+        # Create new socket with correct address family.
+        log.info(
+            "Creating server socket with family=%s and type=%s",
+            self.address_family,
+            self.socket_type)
+        self.socket = socket.socket(self.address_family, self.socket_type)
+
+        super().server_bind()
+
+        # TCPServer.server_bind() overwrites server_address with
+        # socket.getsockname(), which is a tuple of two values when bound to
+        # IPv4 interface (e.g. ('0.0.0.0', 54322)) and tuple of four values
+        # when bound to IPv6 interface (e.g. ('::', 54322, 0, 0)). To make
+        # server address uniform, ensure it's always a tuple of two values -
+        # hostname/IP address and port number.
+        self.server_address = self.server_address[:2]
+
 
 class Connection(BaseHTTPServer.BaseHTTPRequestHandler):
     """
@@ -416,6 +453,13 @@ class Response(object):
         # TODO: return json errors.
         self.status_code = e.code
         body = str(e).encode("utf-8")
+
+        # Content-type header of the response is set to text/plain to mark
+        # content as non-interpretable by the client and thus avoid false
+        # positive from security analysis tools when we send requested URL
+        # back as part of, say, 404 response
+        self.headers["content-type"] = "text/plain; charset=UTF-8"
+
         self.headers["content-length"] = len(body)
         if e.content_range is not None:
             self.headers["content-range"] = e.content_range
