@@ -17,6 +17,7 @@ import socket
 import time
 
 from contextlib import closing
+from contextlib import contextmanager
 
 import six
 from six.moves import http_client
@@ -33,6 +34,8 @@ pytestmark = requires_python3
 
 log = logging.getLogger("test")
 
+ERROR_CONTENT_TYPE = "text/plain; charset=UTF-8"
+
 
 class Demo(object):
 
@@ -304,10 +307,26 @@ class PartialResponse(object):
         raise http.Error(http.INTERNAL_SERVER_ERROR, "No more data for you!")
 
 
+@contextmanager
+def demo_server(address):
+    server = http.Server((address, 0), http.Connection)
+    log.info("Server listening on %r", server.server_address)
+    server.app = http.Router([(r"/demo/(.*)", Demo())])
+
+    t = util.start_thread(
+        server.serve_forever,
+        kwargs={"poll_interval": 0.1})
+    try:
+        yield server
+    finally:
+        server.shutdown()
+        t.join()
+
+
 @pytest.fixture(scope="module")
 def server():
     server = http.Server(("127.0.0.1", 0), http.Connection)
-    log.info("Server listening on port %d", server.server_port)
+    log.info("Server listening on %r", server.server_address)
 
     server.app = http.Router([
         (r"/demo/(.*)", Demo()),
@@ -511,6 +530,7 @@ def test_range_demo(server):
         r = con.getresponse()
         r.read()
         assert r.status == http.REQUESTED_RANGE_NOT_SATISFIABLE
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         assert r.getheader("content-range") == "bytes */16"
 
 
@@ -570,6 +590,7 @@ def test_request_info_get_unsatisfiable_range(server):
         r = con.getresponse()
         r.read()
         assert r.status == http.REQUESTED_RANGE_NOT_SATISFIABLE
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_request_info_put(server):
@@ -604,6 +625,7 @@ def test_request_invalid_content_length(server, content_length):
             headers={"content-length": content_length})
         r = con.getresponse()
         assert r.status == http.BAD_REQUEST
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_request_info_put_content_range(server):
@@ -631,6 +653,7 @@ def test_request_info_put_content_range_invalid(server):
         r = con.getresponse()
         r.read()
         assert r.status == http.BAD_REQUEST
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 @pytest.mark.parametrize("uri,path,arg", [
@@ -684,6 +707,7 @@ def test_context(server):
         con.request("GET", "/context/this")
         r = con.getresponse()
         assert r.status == http.NOT_FOUND
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         r.read()
 
         # Set value for "this".
@@ -708,6 +732,7 @@ def test_context(server):
         con.request("GET", "/context/this")
         r = con.getresponse()
         assert r.status == http.NOT_FOUND
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         r.read()
 
 
@@ -725,6 +750,7 @@ def test_context_per_connection(server):
         con2.request("GET", "/context/this")
         r = con2.getresponse()
         assert r.status == http.NOT_FOUND
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         r.read()
 
         # Set value for "this" in connection 2.
@@ -755,6 +781,7 @@ def test_context_deleted_on_close(server):
         con.request("GET", "/context/this")
         r = con.getresponse()
         assert r.status == http.NOT_FOUND
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         r.read()
 
 
@@ -815,6 +842,7 @@ def test_not_found(server):
         con.request("GET", "/no/such/path")
         r = con.getresponse()
         assert r.status == http.NOT_FOUND
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_method_not_allowed(server):
@@ -823,6 +851,7 @@ def test_method_not_allowed(server):
         con.request("POST", "/demo/name")
         r = con.getresponse()
         assert r.status == http.METHOD_NOT_ALLOWED
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_invalid_method(server):
@@ -831,6 +860,7 @@ def test_invalid_method(server):
         con.request("FOO", "/demo/name")
         r = con.getresponse()
         assert r.status == http.METHOD_NOT_ALLOWED
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_client_error_get(server):
@@ -839,6 +869,7 @@ def test_client_error_get(server):
         con.request("GET", "/client-error/")
         r = con.getresponse()
         assert r.status == http.FORBIDDEN
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_client_error_put(server):
@@ -851,6 +882,7 @@ def test_client_error_put(server):
                 raise
         r = con.getresponse()
         assert r.status == http.FORBIDDEN
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_internal_error_get(server):
@@ -860,6 +892,7 @@ def test_internal_error_get(server):
         con.request("GET", "/server-error/")
         r = con.getresponse()
         assert r.status == http.INTERNAL_SERVER_ERROR
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         assert "secret" not in r.read().decode("utf-8")
 
 
@@ -874,6 +907,7 @@ def test_internal_error_put(server):
                 raise
         r = con.getresponse()
         assert r.status == http.INTERNAL_SERVER_ERROR
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
         assert "secret" not in r.read().decode("utf-8")
 
 
@@ -884,6 +918,7 @@ def test_server_socket_error_get(server):
         con.request("GET", "/server-socket-error/")
         r = con.getresponse()
         assert r.status == http.INTERNAL_SERVER_ERROR
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_server_socket_error_put(server):
@@ -897,6 +932,7 @@ def test_server_socket_error_put(server):
                 raise
         r = con.getresponse()
         assert r.status == http.INTERNAL_SERVER_ERROR
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 @pytest.mark.parametrize("data", [None, b"", b"read me"])
@@ -916,6 +952,7 @@ def test_keep_connection_on_error(server, data):
             r = con.getresponse()
             r.read()
             assert r.status == http.FORBIDDEN
+            assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
 
 def test_close_connection_on_error(server):
@@ -933,6 +970,7 @@ def test_close_connection_on_error(server):
         r = con.getresponse()
         r.read()
         assert r.status == http.FORBIDDEN
+        assert r.getheader("content-type") == ERROR_CONTENT_TYPE
 
         # Try to send another request. This will fail since we disabled
         # auto_open.  Fails in request() or in getresponse(), probably
@@ -1064,3 +1102,64 @@ def test_content_range_parse_invalid(header):
     with pytest.raises(http.Error) as e:
         http.ContentRange.parse(header)
     assert e.value.code == http.BAD_REQUEST
+
+
+@pytest.mark.parametrize("listen_address", [
+    "localhost",
+    "127.0.0.1",
+    "::1",
+])
+def test_server_bind_loopback(listen_address):
+    with demo_server(listen_address) as server:
+        # TCPServer returns tuple of different length for IPv4 and IPv6. We
+        # change server_address in Server.server_bind() to be always tuple
+        # of hostname and port. Check it here, that it has always two elements.
+        assert len(server.server_address) == 2
+
+        con = http_client.HTTPConnection("localhost", server.server_port)
+        with closing(con):
+            con.request("GET", "/demo/name")
+            con.getresponse()
+
+
+def test_server_bind_ipv4():
+    with demo_server("") as server:
+        # Connecting via IPv4 address should work.
+        con = http_client.HTTPConnection("127.0.0.1", server.server_port)
+        with closing(con):
+            con.request("GET", "/demo/name")
+            con.getresponse()
+
+        # Connecting via IPv6 address should fail.
+        con = http_client.HTTPConnection("::1", server.server_port)
+        with closing(con):
+            with pytest.raises(ConnectionRefusedError):
+                con.request("GET", "/demo/name")
+
+
+@pytest.mark.parametrize("connect_address", [
+    "127.0.0.1",
+    "::1",
+    "localhost",
+    socket.gethostname(),
+])
+def test_server_bind_dual_stack(connect_address):
+    with demo_server("::") as server:
+        con = http_client.HTTPConnection(connect_address, server.server_port)
+        with closing(con):
+            con.request("GET", "/demo/name")
+            con.getresponse()
+
+
+def test_server_bind_addr_from_hostname():
+    infos = socket.getaddrinfo(
+        socket.gethostname(),
+        0,
+        type=socket.SOCK_STREAM)
+    for info in infos:
+        address = info[4][0]
+        with demo_server(address) as server:
+            con = http_client.HTTPConnection(address, server.server_port)
+            with closing(con):
+                con.request("GET", "/demo/name")
+                con.getresponse()
